
#include <pylon/PylonIncludes.h>
#include <stdlib.h>

using namespace Pylon;

#include <pylon/gige/BaslerGigECamera.h>
typedef Pylon::CBaslerGigECamera Camera_t;
using namespace Basler_GigECameraParams;
using namespace Basler_GigEStreamParams;

using namespace std;

void pressEnterToExit()
{
    cerr << endl << "Press enter to exit." << endl;
    while( cin.get() != '\n');
}

class CGrabBuffer
{
    public:
        CGrabBuffer(const size_t ImageSize);
        ~CGrabBuffer();
        uint8_t* GetBufferPointer(void) { return m_pBuffer; }
        StreamBufferHandle GetBufferHandle(void) { return m_hBuffer; }
        void SetBufferHandle(StreamBufferHandle hBuffer) { m_hBuffer = hBuffer; };

    protected:
        uint8_t *m_pBuffer;
        StreamBufferHandle m_hBuffer;
};

CGrabBuffer::CGrabBuffer(const size_t ImageSize):
        m_pBuffer(NULL)
{
    m_pBuffer = new uint8_t[ ImageSize ];
    if (NULL == m_pBuffer)
    {
        GenICam::GenericException e("Not enough memory to allocate image buffer", __FILE__, __LINE__);
        throw e;
    }
}

CGrabBuffer::~CGrabBuffer()
{
    if (NULL != m_pBuffer)
        delete[] m_pBuffer;
}


static const uint32_t c_MultiShotCount = 100;

int main(int argc, char* argv[])
{
    Pylon::PylonAutoInitTerm autoInitTerm;

    try
    {
		//	GigEp̃gX|[gC[̐
        CTlFactory& TlFactory = CTlFactory::GetInstance();
        ITransportLayer *pTl = TlFactory.CreateTl(Camera_t::DeviceClass());
        if (! pTl)
        {
            cerr << "Failed to create transport layer!" << endl;
            pressEnterToExit();
            return 1;
        }

		//	ڑĂJ邩ǂ̃`FbN
        DeviceInfoList_t devices;
        if (0 == pTl->EnumerateDevices(devices))
        {
            cerr << "No camera present!" << endl;
            pressEnterToExit();
            return 1;
        }

		//	JI[v
        Camera_t Camera(pTl->CreateDevice(devices[ 0 ]));
        Camera.Open();

		//	Xg[I[v
        Camera_t::StreamGrabber_t StreamGrabber(Camera.GetStreamGrabber(0));
        StreamGrabber.Open();

		//	8bitmNɕύX
        Camera.PixelFormat.SetValue(PixelFormat_Mono8);
		//	tt[ɕύX
        Camera.OffsetX.SetValue(0);
        Camera.OffsetY.SetValue(0);
        Camera.Width.SetValue(Camera.Width.GetMax());
        Camera.Height.SetValue(Camera.Height.GetMax());

        //	FrameStartgK[𖳌ɂ
        {
            GenApi::IEnumEntry* frameStart = Camera.TriggerSelector.GetEntry( TriggerSelector_FrameStart);
            if ( frameStart && GenApi::IsAvailable( frameStart))
            {
                Camera.TriggerSelector.SetValue( TriggerSelector_FrameStart);
                Camera.TriggerMode.SetValue( TriggerMode_Off);
            }
        }

		//	AcqusitionStartgK[Lɂ
		{
            GenApi::IEnumEntry* acquisitionStart = Camera.TriggerSelector.GetEntry( TriggerSelector_AcquisitionStart);
            if ( acquisitionStart && GenApi::IsAvailable( acquisitionStart))
            {
                Camera.TriggerSelector.SetValue( TriggerSelector_AcquisitionStart);
                Camera.TriggerMode.SetValue( TriggerMode_On);
            }
        }

		//	\tgEFAgK[Lɂ
		Camera.TriggerSource.SetValue( TriggerSource_Software );

		//	}`VbgLɂ
		Camera.AcquisitionFrameCount.SetValue( c_MultiShotCount );

        //	AcquisitionModeContinuousɕύX
        Camera.AcquisitionMode.SetValue(AcquisitionMode_Continuous);

        //	IԂ̐ݒύX
        Camera.ExposureMode.SetValue(ExposureMode_Timed);
        Camera.ExposureAuto.SetValue(ExposureAuto_Off);
        Camera.ExposureTimeAbs.SetValue(1000);

		//	捞TCYуobt@ݒ肷
        const size_t ImageSize = (size_t)(Camera.PayloadSize.GetValue());
        StreamGrabber.MaxBufferSize.SetValue(ImageSize);
        StreamGrabber.MaxNumBuffer.SetValue(c_MultiShotCount);

        // Allocate all resources for grabbing. Critical parameters like image
        // size now must not be changed until FinishGrab() is called.
		//	捞ɕKvȃ\[X
		//	ȍ~A摜̕AȂǂ̒l͕ύXł܂B
		//	TChύX邽߂ɂ́AFinishGrabsKv܂B
        StreamGrabber.PrepareGrab();

		//	捞obt@mۂ
        std::vector<CGrabBuffer*> BufferList;
        for (uint32_t i = 0; i < c_MultiShotCount; ++i)
        {
            CGrabBuffer *pGrabBuffer = new CGrabBuffer(ImageSize);
            pGrabBuffer->SetBufferHandle(StreamGrabber.RegisterBuffer(
                pGrabBuffer->GetBufferPointer(), ImageSize));

            BufferList.push_back(pGrabBuffer);
        }

		//	捞JnII
		Camera.AcquisitionStart.Execute();

		//	}`Vbg3s
		for( int m = 0; m < 3; ++m )
		{
			//	捞obt@̃L[COs
			for (std::vector<CGrabBuffer*>::const_iterator x = BufferList.begin(); x != BufferList.end(); ++x)
			{
				StreamGrabber.QueueBuffer((*x)->GetBufferHandle(), NULL);
			}

			//	\tgEFAgK[s
			Camera.TriggerSoftware.Execute();

			//	荞100̉摜擾
			for (int n = 0; n < c_MultiShotCount; n++)
			{
				//	捞҂
				if (StreamGrabber.GetWaitObject().Wait(3000))
				{
					GrabResult Result;
					StreamGrabber.RetrieveResult(Result);

					if (Grabbed == Result.Status())
					{
						cout << "Image #" << m+1 << "-" << n << " acquired!" << endl;
						cout << "Size: " << Result.GetSizeX() << " x " << Result.GetSizeY() << endl;

						//	[̋Px\
						const uint8_t *pImageBuffer = (uint8_t *) Result.Buffer();
						cout << "Gray value of first pixel: " << (uint32_t) pImageBuffer[0] << endl << endl;
					}
					else if (Failed == Result.Status())
					{
						//	G[
						cerr << "No image acquired!" << endl;
						cerr << "Error code : 0x" << hex << Result.GetErrorCode() << endl;
						cerr << "Error description : " << Result.GetErrorDescription() << endl;
					}
				}
				else
				{
					//	^CAEg
					cerr << "Timeout occurred!" << endl;

					//	݃L[COĂobt@ׂēfo
					StreamGrabber.CancelGrab();
					for (GrabResult r; StreamGrabber.RetrieveResult(r););

					// Cancel loop
					break;
				}
			}

		}

        //	捞~
        Camera.AcquisitionStop.Execute();

        //	N[Abv

        for (std::vector<CGrabBuffer*>::iterator it = BufferList.begin(); it != BufferList.end(); it++)
        {
            StreamGrabber.DeregisterBuffer((*it)->GetBufferHandle());
            delete *it;
            *it = NULL;
        }

        StreamGrabber.FinishGrab();

        StreamGrabber.Close();

        Camera.Close();

    }
    catch (GenICam::GenericException &e)
    {
        //	G[
        cerr << "An exception occurred!" << endl

        << e.GetDescription() << endl;
        pressEnterToExit();
        return 1;
    }

	pressEnterToExit();

	return 0;
}

